#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>

#define VERSION "0.2.0"
#define CMD_MAX_LEN 45

static int interval = 5;

static void
usage()
{
	printf("%s: %s [OPTIONS] COMMAND\n"
	       "will repeatedly run COMMAND every %d seconds.\n"
	       "Following OPTIONS are valid:\n"
	       "  -v          show version\n"
	       "  -n SECONDS  set interval between command invocations\n"
	       "  -r N        repeat for N times and then quit\n"
	       "  -q          suppress output (stdout) from COMMAND\n"
	       "  -c          do not clear screen between COMMAND calls\n"
	     "  --          do not consider any following args as options\n"
	       ,
	       __func__, getprogname(), interval);
}

static void
die(const char *errstr,...)
{
	va_list ap;

	va_start(ap, errstr);
	vfprintf(stderr, errstr, ap);
	va_end(ap);
	exit(1);
}

static void
run_cmd(int sleeptime, char *command, int quiet, int clearscreen)
{
	if (!quiet) {
		if (clearscreen)
			printf("\e[1;1H\e[2J");	/* clear screen (POSIX) */
		else
			puts("");
		printf("Every %ds: %s\n\n", sleeptime, command);
	}
	if (quiet)
		strcat(command, " >/dev/null");
	system(command);
	sleep(sleeptime);
}

int
main(int argc, char *argv[])
{
	int i, opt;
	int repeat = 0, quiet = 0, clearscreen = 1;
	char command[CMD_MAX_LEN] = "";

	while ((opt = getopt(argc, argv, "hvn:r:qc")) != -1) {
		switch (opt) {
		case -1:	/* no more arguments */
		case 0:	/* long options toggles */
			break;
		case 'h':
			usage();
			return 0;
			break;
		case 'v':
			die("%s " VERSION "\n", argv[0]);
			break;
		case 'n':
			interval = atoi(optarg);
			break;
		case 'r':
			repeat = atoi(optarg);
			break;
		case 'q':
			quiet = 1;
			break;
		case 'c':
			clearscreen = 0;
			break;
		default:
			fprintf(stderr, "%s: invalid option -- %c\n", argv[0], opt);
			fprintf(stderr, "try `%s --help` for more information\n",
				argv[0]);
			return -2;
		}
	}
	argc -= optind;
	argv += optind;

	if (argc < 1)
		die("error: no COMMAND specified\n");

	strcat(command, "[ -f ~/.profile ] && . ~/.profile; ");
	for (i = 0; i < argc; ++i) {
		if (strlen(command) + strlen(argv[i]) +
			(i > 0 && i < argc - 1 ? 1 : 0) >= CMD_MAX_LEN)
			die("error: command length exceeded\n");

		strcat(command, argv[i]);
		if (i > 0 && i < argc - 1)
			strcat(command, " ");
	}

	if (repeat > 0) {
		for (i = 0; i < repeat; ++i)
			run_cmd(interval, command, quiet, clearscreen);
	} else {
		for (;;)
			run_cmd(interval, command, quiet, clearscreen);
	}
	return 0;
}
